package javax.swing;

import java.util.ArrayList;
import sun.awt.AWTAccessor;
import java.awt.event.KeyEvent;
import java.awt.KeyEventPostProcessor;
import sun.awt.PaintEventDispatcher;
import java.awt.KeyboardFocusManager;
import sun.awt.AppContext;
import java.util.HashMap;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.io.FileInputStream;
import java.security.PrivilegedAction;
import java.util.Properties;
import java.beans.PropertyChangeListener;
import java.util.Vector;
import javax.swing.plaf.ComponentUI;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.Component;
import javax.swing.border.Border;
import java.awt.Color;
import java.util.Locale;
import java.awt.Font;
import sun.awt.SunToolkit;
import java.awt.Toolkit;
import sun.awt.OSInfo;
import java.security.AccessController;
import sun.security.action.GetPropertyAction;
import javax.swing.plaf.metal.MetalLookAndFeel;
import javax.swing.event.SwingPropertyChangeSupport;
import sun.swing.DefaultLookup;
import java.io.File;
import sun.swing.SwingUtilities2;
import java.io.Serializable;

public class UIManager implements Serializable
{
    private static final Object classLock;
    private static final String defaultLAFKey = "swing.defaultlaf";
    private static final String auxiliaryLAFsKey = "swing.auxiliarylaf";
    private static final String multiplexingLAFKey = "swing.plaf.multiplexinglaf";
    private static final String installedLAFsKey = "swing.installedlafs";
    private static final String disableMnemonicKey = "swing.disablenavaids";
    private static UIManager.LookAndFeelInfo[] installedLAFs;
    
    public UIManager() {
    }
    
    private static UIManager.LAFState getLAFState() {
        UIManager.LAFState lafState = (UIManager.LAFState)SwingUtilities.appContextGet(SwingUtilities2.LAF_STATE_KEY);
        if (lafState == null) {
            synchronized (UIManager.classLock) {
                lafState = (UIManager.LAFState)SwingUtilities.appContextGet(SwingUtilities2.LAF_STATE_KEY);
                if (lafState == null) {
                    SwingUtilities.appContextPut(SwingUtilities2.LAF_STATE_KEY, lafState = new UIManager.LAFState());
                }
            }
        }
        return lafState;
    }
    
    private static String makeInstalledLAFKey(String str, String str2) {
        return "swing.installedlaf." + str + "." + str2;
    }
    
    private static String makeSwingPropertiesFilename() {
        String separator = File.separator;
        String property = System.getProperty("java.home");
        if (property == null) {
            property = "<java.home undefined>";
        }
        return property + separator + "lib" + separator + "swing.properties";
    }
    
    public static UIManager.LookAndFeelInfo[] getInstalledLookAndFeels() {
        maybeInitialize();
        UIManager.LookAndFeelInfo[] array = getLAFState().installedLAFs;
        if (array == null) {
            array = UIManager.installedLAFs;
        }
        UIManager.LookAndFeelInfo[] array2 = new UIManager.LookAndFeelInfo[array.length];
        System.arraycopy(array, 0, array2, 0, array.length);
        return array2;
    }
    
    public static void setInstalledLookAndFeels(UIManager.LookAndFeelInfo[] array) throws SecurityException {
        maybeInitialize();
        UIManager.LookAndFeelInfo[] installedLAFs = new UIManager.LookAndFeelInfo[array.length];
        System.arraycopy(array, 0, installedLAFs, 0, array.length);
        getLAFState().installedLAFs = installedLAFs;
    }
    
    public static void installLookAndFeel(UIManager.LookAndFeelInfo lookAndFeelInfo) {
        UIManager.LookAndFeelInfo[] installedLookAndFeels = getInstalledLookAndFeels();
        UIManager.LookAndFeelInfo[] installedLookAndFeels2 = new UIManager.LookAndFeelInfo[installedLookAndFeels.length + 1];
        System.arraycopy(installedLookAndFeels, 0, installedLookAndFeels2, 0, installedLookAndFeels.length);
        installedLookAndFeels2[installedLookAndFeels.length] = lookAndFeelInfo;
        setInstalledLookAndFeels(installedLookAndFeels2);
    }
    
    public static void installLookAndFeel(String s, String s2) {
        installLookAndFeel(new LookAndFeelInfo(s, s2));
    }
    
    public static LookAndFeel getLookAndFeel() {
        maybeInitialize();
        return getLAFState().lookAndFeel;
    }
    
    public static void setLookAndFeel(LookAndFeel lookAndFeel) throws UnsupportedLookAndFeelException {
        if (lookAndFeel != null && !lookAndFeel.isSupportedLookAndFeel()) {
            throw new UnsupportedLookAndFeelException(lookAndFeel.toString() + " not supported on this platform");
        }
        UIManager.LAFState lafState = getLAFState();
        LookAndFeel lookAndFeel2 = lafState.lookAndFeel;
        if (lookAndFeel2 != null) {
            lookAndFeel2.uninitialize();
        }
        if ((lafState.lookAndFeel = lookAndFeel) != null) {
            DefaultLookup.setDefaultLookup(null);
            lookAndFeel.initialize();
            lafState.setLookAndFeelDefaults(lookAndFeel.getDefaults());
        }
        else {
            lafState.setLookAndFeelDefaults(null);
        }
        SwingPropertyChangeSupport propertyChangeSupport = lafState.getPropertyChangeSupport(false);
        if (propertyChangeSupport != null) {
            propertyChangeSupport.firePropertyChange("lookAndFeel", lookAndFeel2, lookAndFeel);
        }
        return;
    }
    
    public static void setLookAndFeel(String anObject) throws ClassNotFoundException, InstantiationException, IllegalAccessException, UnsupportedLookAndFeelException {
        if ("javax.swing.plaf.metal.MetalLookAndFeel".equals(anObject)) {
            setLookAndFeel(new MetalLookAndFeel());
        }
        else {
            setLookAndFeel((LookAndFeel)(LookAndFeel)SwingUtilities.loadSystemClass(anObject).newInstance());
        }
    }
    
    public static String getSystemLookAndFeelClassName() {
        String s = (String)AccessController.doPrivileged(new GetPropertyAction("swing.systemlaf"));
        if (s != null) {
            return s;
        }
        OSInfo.OSType osType = (OSInfo.OSType)AccessController.doPrivileged(OSInfo.getOSTypeAction());
        if (osType == OSInfo.OSType.WINDOWS) {
            return "com.sun.java.swing.plaf.windows.WindowsLookAndFeel";
        }
        String anObject = (String)AccessController.doPrivileged(new GetPropertyAction("sun.desktop"));
        Toolkit defaultToolkit = Toolkit.getDefaultToolkit();
        if ("gnome".equals(anObject) && defaultToolkit instanceof SunToolkit && ((SunToolkit)defaultToolkit).isNativeGTKAvailable()) {
            return "com.sun.java.swing.plaf.gtk.GTKLookAndFeel";
        }
        if (osType == OSInfo.OSType.MACOSX && defaultToolkit.getClass().getName().equals("sun.lwawt.macosx.LWCToolkit")) {
            return "com.apple.laf.AquaLookAndFeel";
        }
        if (osType == OSInfo.OSType.SOLARIS) {
            return "com.sun.java.swing.plaf.motif.MotifLookAndFeel";
        }
        return getCrossPlatformLookAndFeelClassName();
    }
    
    public static String getCrossPlatformLookAndFeelClassName() {
        String s = (String)AccessController.doPrivileged(new GetPropertyAction("swing.crossplatformlaf"));
        if (s != null) {
            return s;
        }
        return "javax.swing.plaf.metal.MetalLookAndFeel";
    }
    
    public static UIDefaults getDefaults() {
        maybeInitialize();
        return getLAFState().multiUIDefaults;
    }
    
    public static Font getFont(Object o) {
        return getDefaults().getFont(o);
    }
    
    public static Font getFont(Object o, Locale locale) {
        return getDefaults().getFont(o, locale);
    }
    
    public static Color getColor(Object o) {
        return getDefaults().getColor(o);
    }
    
    public static Color getColor(Object o, Locale locale) {
        return getDefaults().getColor(o, locale);
    }
    
    public static Icon getIcon(Object o) {
        return getDefaults().getIcon(o);
    }
    
    public static Icon getIcon(Object o, Locale locale) {
        return getDefaults().getIcon(o, locale);
    }
    
    public static Border getBorder(Object o) {
        return getDefaults().getBorder(o);
    }
    
    public static Border getBorder(Object o, Locale locale) {
        return getDefaults().getBorder(o, locale);
    }
    
    public static String getString(Object o) {
        return getDefaults().getString(o);
    }
    
    public static String getString(Object o, Locale locale) {
        return getDefaults().getString(o, locale);
    }
    
    static String getString(Object o, Component component) {
        return getString(o, (component == null) ? Locale.getDefault() : component.getLocale());
    }
    
    public static int getInt(Object o) {
        return getDefaults().getInt(o);
    }
    
    public static int getInt(Object o, Locale locale) {
        return getDefaults().getInt(o, locale);
    }
    
    public static boolean getBoolean(Object o) {
        return getDefaults().getBoolean(o);
    }
    
    public static boolean getBoolean(Object o, Locale locale) {
        return getDefaults().getBoolean(o, locale);
    }
    
    public static Insets getInsets(Object o) {
        return getDefaults().getInsets(o);
    }
    
    public static Insets getInsets(Object o, Locale locale) {
        return getDefaults().getInsets(o, locale);
    }
    
    public static Dimension getDimension(Object o) {
        return getDefaults().getDimension(o);
    }
    
    public static Dimension getDimension(Object o, Locale locale) {
        return getDefaults().getDimension(o, locale);
    }
    
    public static Object get(Object o) {
        return getDefaults().get(o);
    }
    
    public static Object get(Object o, Locale locale) {
        return getDefaults().get(o, locale);
    }
    
    public static Object put(Object o, Object o2) {
        return getDefaults().put(o, o2);
    }
    
    public static ComponentUI getUI(JComponent component) {
        maybeInitialize();
        maybeInitializeFocusPolicy(component);
        ComponentUI componentUI = null;
        LookAndFeel multiLookAndFeel = getLAFState().multiLookAndFeel;
        if (multiLookAndFeel != null) {
            componentUI = multiLookAndFeel.getDefaults().getUI(component);
        }
        if (componentUI == null) {
            componentUI = getDefaults().getUI(component);
            return componentUI;
        }
        return componentUI;
    }
    
    public static UIDefaults getLookAndFeelDefaults() {
        maybeInitialize();
        return getLAFState().getLookAndFeelDefaults();
    }
    
    private static LookAndFeel getMultiLookAndFeel() {
        LookAndFeel multiLookAndFeel = getLAFState().multiLookAndFeel;
        if (multiLookAndFeel == null) {
            String property = getLAFState().swingProps.getProperty("swing.plaf.multiplexinglaf", "javax.swing.plaf.multi.MultiLookAndFeel");
            try {
                multiLookAndFeel = (LookAndFeel)SwingUtilities.loadSystemClass(property).newInstance();
            }
            catch (Exception ex) {
                System.err.println("UIManager: failed loading " + property);
            }
        }
        return multiLookAndFeel;
    }
    
    public static void addAuxiliaryLookAndFeel(LookAndFeel lookAndFeel) {
        maybeInitialize();
        if (!lookAndFeel.isSupportedLookAndFeel()) {
            return;
        }
        Vector<LookAndFeel> auxLookAndFeels = getLAFState().auxLookAndFeels;
        if (auxLookAndFeels == null) {
            auxLookAndFeels = new Vector();
        }
        if (!auxLookAndFeels.contains(lookAndFeel)) {
            auxLookAndFeels.addElement(lookAndFeel);
            lookAndFeel.initialize();
            getLAFState().auxLookAndFeels = auxLookAndFeels;
            if (getLAFState().multiLookAndFeel == null) {
                getLAFState().multiLookAndFeel = getMultiLookAndFeel();
            }
        }
        return;
    }
    
    public static boolean removeAuxiliaryLookAndFeel(LookAndFeel obj) {
        maybeInitialize();
        Vector<Object> auxLookAndFeels = getLAFState().auxLookAndFeels;
        if (auxLookAndFeels == null || auxLookAndFeels.size() == 0) {
            return false;
        }
        int removeElement = auxLookAndFeels.removeElement(obj);
        if (!(removeElement == 0)) {
            if (auxLookAndFeels.size() == 0) {
                getLAFState().auxLookAndFeels = null;
                getLAFState().multiLookAndFeel = null;
            }
            else {
                getLAFState().auxLookAndFeels = auxLookAndFeels;
            }
        }
        obj.uninitialize();
        return removeElement;
    }
    
    public static LookAndFeel[] getAuxiliaryLookAndFeels() {
        maybeInitialize();
        Vector<Object> auxLookAndFeels = getLAFState().auxLookAndFeels;
        if (auxLookAndFeels == null || auxLookAndFeels.size() == 0) {
            return null;
        }
        LookAndFeel[] array = new LookAndFeel[auxLookAndFeels.size()];
        for (int i = 0; i < array.length; ++i) {
            array[i] = (LookAndFeel)auxLookAndFeels.elementAt(i);
        }
        return array;
    }
    
    public static void addPropertyChangeListener(PropertyChangeListener listener) {
        synchronized (UIManager.classLock) {
            getLAFState().getPropertyChangeSupport(true).addPropertyChangeListener(listener);
        }
    }
    
    public static void removePropertyChangeListener(PropertyChangeListener listener) {
        synchronized (UIManager.classLock) {
            getLAFState().getPropertyChangeSupport(true).removePropertyChangeListener(listener);
        }
    }
    
    public static PropertyChangeListener[] getPropertyChangeListeners() {
        synchronized (UIManager.classLock) {
            return getLAFState().getPropertyChangeSupport(true).getPropertyChangeListeners();
        }
    }
    
    private static Properties loadSwingProperties() {
        if (UIManager.class.getClassLoader() != null) {
            return new Properties();
        }
        final Properties properties = new Properties();
        AccessController.doPrivileged(new PrivilegedAction<Object>() {
            @Override
            public Object run() {
                if ((OSInfo.OSType)AccessController.doPrivileged(OSInfo.getOSTypeAction()) == OSInfo.OSType.MACOSX) {
                    properties.put("swing.defaultlaf", UIManager.getSystemLookAndFeelClassName());
                }
                try {
                    File file = new File(UIManager.access$100());
                    if (file.exists()) {
                        FileInputStream inStream = new FileInputStream(file);
                        properties.load(inStream);
                        inStream.close();
                    }
                }
                catch (Exception ex) {}
                UIManager.access$200(properties, "swing.defaultlaf");
                UIManager.access$200(properties, "swing.auxiliarylaf");
                UIManager.access$200(properties, "swing.plaf.multiplexinglaf");
                UIManager.access$200(properties, "swing.installedlafs");
                UIManager.access$200(properties, "swing.disablenavaids");
                return null;
            }
        });
        return properties;
    }
    
    private static void checkProperty(Properties properties, String s) {
        String property = System.getProperty(s);
        if (property != null) {
            properties.put(s, property);
        }
    }
    
    private static void initializeInstalledLAFs(Properties properties) {
        String property = properties.getProperty("swing.installedlafs");
        if (property == null) {
            return;
        }
        Vector<String> vector = new Vector();
        StringTokenizer stringTokenizer = new StringTokenizer(property, ",", false);
        while (stringTokenizer.hasMoreTokens()) {
            vector.addElement(stringTokenizer.nextToken());
        }
        Vector<UIManager.LookAndFeelInfo> vector2 = new Vector(vector.size());
        for (String defaultValue : vector) {
            String property2 = properties.getProperty(makeInstalledLAFKey(defaultValue, "name"), defaultValue);
            String property3 = properties.getProperty(makeInstalledLAFKey(defaultValue, "class"));
            if (property3 != null) {
                vector2.addElement(new LookAndFeelInfo(property2, property3));
            }
            else {
                continue;
            }
        }
        UIManager.LookAndFeelInfo[] installedLAFs = new UIManager.LookAndFeelInfo[vector2.size()];
        for (int i = 0; i < vector2.size(); ++i) {
            installedLAFs[i] = (UIManager.LookAndFeelInfo)vector2.elementAt(i);
        }
        getLAFState().installedLAFs = installedLAFs;
        return;
    }
    
    private static void initializeDefaultLAF(Properties properties) {
        if (getLAFState().lookAndFeel != null) {
            return;
        }
        String crossPlatformLookAndFeelClassName = null;
        HashMap<Object, Object> hashMap = (HashMap)AppContext.getAppContext().remove("swing.lafdata");
        if (hashMap != null) {
            crossPlatformLookAndFeelClassName = (String)hashMap.remove("defaultlaf");
        }
        if (crossPlatformLookAndFeelClassName == null) {
            crossPlatformLookAndFeelClassName = getCrossPlatformLookAndFeelClassName();
        }
        String property = properties.getProperty("swing.defaultlaf", crossPlatformLookAndFeelClassName);
        try {
            setLookAndFeel(property);
        }
        catch (Exception ex) {
            throw new Error("Cannot load " + property);
        }
        if (hashMap != null) {
            for (Object next : hashMap.keySet()) {
                put(next, hashMap.get(next));
            }
        }
        return;
    }
    
    private static void initializeAuxiliaryLAFs(Properties properties) {
        String property = properties.getProperty("swing.auxiliarylaf");
        if (property == null) {
            return;
        }
        Vector<LookAndFeel> auxLookAndFeels = new Vector();
        StringTokenizer stringTokenizer = new StringTokenizer(property, ",");
        while (stringTokenizer.hasMoreTokens()) {
            String nextToken = stringTokenizer.nextToken();
            try {
                LookAndFeel obj = (LookAndFeel)SwingUtilities.loadSystemClass(nextToken).newInstance();
                obj.initialize();
                auxLookAndFeels.addElement(obj);
            }
            catch (Exception ex) {
                System.err.println("UIManager: failed loading auxiliary look and feel " + nextToken);
            }
        }
        if (auxLookAndFeels.size() == 0) {
            auxLookAndFeels = null;
        }
        else {
            getLAFState().multiLookAndFeel = getMultiLookAndFeel();
            if (getLAFState().multiLookAndFeel == null) {
                auxLookAndFeels = null;
            }
        }
        getLAFState().auxLookAndFeels = auxLookAndFeels;
        return;
    }
    
    private static void initializeSystemDefaults(Properties swingProps) {
        getLAFState().swingProps = swingProps;
    }
    
    private static void maybeInitialize() {
        synchronized (UIManager.classLock) {
            if (!getLAFState().initialized) {
                getLAFState().initialized = true;
                initialize();
            }
        }
    }
    
    private static void maybeInitializeFocusPolicy(JComponent component) {
        if ((component instanceof JRootPane)) {
            synchronized (UIManager.classLock) {
                if (!getLAFState().focusPolicyInitialized) {
                    getLAFState().focusPolicyInitialized = true;
                    if (FocusManager.isFocusManagerEnabled()) {
                        KeyboardFocusManager.getCurrentKeyboardFocusManager().setDefaultFocusTraversalPolicy(new LayoutFocusTraversalPolicy());
                    }
                }
            }
        }
    }
    
    private static void initialize() {
        Properties loadSwingProperties = loadSwingProperties();
        initializeSystemDefaults(loadSwingProperties);
        initializeDefaultLAF(loadSwingProperties);
        initializeAuxiliaryLAFs(loadSwingProperties);
        initializeInstalledLAFs(loadSwingProperties);
        if (RepaintManager.HANDLE_TOP_LEVEL_PAINT) {
            PaintEventDispatcher.setPaintEventDispatcher(new SwingPaintEventDispatcher());
        }
        KeyboardFocusManager.getCurrentKeyboardFocusManager().addKeyEventPostProcessor(new KeyEventPostProcessor() {
            @Override
            public boolean postProcessKeyEvent(KeyEvent keyEvent) {
                Component component = keyEvent.getComponent();
                if ((!(component instanceof JComponent) || (component != null && !component.isEnabled())) && JComponent.KeyboardState.shouldProcess(keyEvent) && SwingUtilities.processKeyBindings(keyEvent)) {
                    keyEvent.consume();
                    return true;
                }
                return false;
            }
        });
        AWTAccessor.getComponentAccessor().setRequestFocusController(JComponent.focusController);
    }
    
    static String access$100() {
        return makeSwingPropertiesFilename();
    }
    
    static void access$200(Properties properties, String s) {
        checkProperty(properties, s);
    }
    
    static {
        classLock = new Object();
        ArrayList list = new ArrayList(4);
        list.add(new UIManager.LookAndFeelInfo("Metal", "javax.swing.plaf.metal.MetalLookAndFeel"));
        list.add(new UIManager.LookAndFeelInfo("Nimbus", "javax.swing.plaf.nimbus.NimbusLookAndFeel"));
        list.add(new UIManager.LookAndFeelInfo("CDE/Motif", "com.sun.java.swing.plaf.motif.MotifLookAndFeel"));
        OSInfo.OSType osType = (OSInfo.OSType)AccessController.doPrivileged(OSInfo.getOSTypeAction());
        if (osType == OSInfo.OSType.WINDOWS) {
            list.add(new UIManager.LookAndFeelInfo("Windows", "com.sun.java.swing.plaf.windows.WindowsLookAndFeel"));
            if (Toolkit.getDefaultToolkit().getDesktopProperty("win.xpstyle.themeActive") != null) {
                list.add(new UIManager.LookAndFeelInfo("Windows Classic", "com.sun.java.swing.plaf.windows.WindowsClassicLookAndFeel"));
            }
        }
        else if (osType == OSInfo.OSType.MACOSX) {
            list.add(new UIManager.LookAndFeelInfo("Mac OS X", "com.apple.laf.AquaLookAndFeel"));
        }
        else {
            list.add(new UIManager.LookAndFeelInfo("GTK+", "com.sun.java.swing.plaf.gtk.GTKLookAndFeel"));
        }
        UIManager.installedLAFs = (UIManager.LookAndFeelInfo[])list.toArray(new UIManager.LookAndFeelInfo[list.size()]);
    }
    
    private static class LAFState
    {
        Properties swingProps;
        private UIDefaults[] tables;
        boolean initialized;
        boolean focusPolicyInitialized;
        MultiUIDefaults multiUIDefaults;
        LookAndFeel lookAndFeel;
        LookAndFeel multiLookAndFeel;
        Vector<LookAndFeel> auxLookAndFeels;
        SwingPropertyChangeSupport changeSupport;
        UIManager.LookAndFeelInfo[] installedLAFs;
        
        private LAFState() {
            this.tables = new UIDefaults[2];
            this.initialized = false;
            this.focusPolicyInitialized = false;
            this.multiUIDefaults = new MultiUIDefaults(this.tables);
            this.multiLookAndFeel = null;
            this.auxLookAndFeels = null;
        }
        
        UIDefaults getLookAndFeelDefaults() {
            return this.tables[0];
        }
        
        void setLookAndFeelDefaults(UIDefaults uiDefaults) {
            this.tables[0] = uiDefaults;
        }
        
        UIDefaults getSystemDefaults() {
            return this.tables[1];
        }
        
        void setSystemDefaults(UIDefaults uiDefaults) {
            this.tables[1] = uiDefaults;
        }
        
        public synchronized SwingPropertyChangeSupport getPropertyChangeSupport(boolean b) {
            if (b && this.changeSupport == null) {
                this.changeSupport = new SwingPropertyChangeSupport(UIManager.class);
            }
            return this.changeSupport;
        }
    }
    
    public static class LookAndFeelInfo
    {
        private String name;
        private String className;
        
        public LookAndFeelInfo(String name, String className) {
            this.name = name;
            this.className = className;
        }
        
        public String getName() {
            return this.name;
        }
        
        public String getClassName() {
            return this.className;
        }
        
        @Override
        public String toString() {
            return this.getClass().getName() + "[" + this.getName() + " " + this.getClassName() + "]";
        }
    }
}
